# -*- coding: iso-8859-15 -*-

# Contractor, a contract validation tool
#    Copyright (C) 2009  Guido de Caso, Alexis Tcach, Víctor Braberman, Diego Garbervetsky, Sebastián Uchitel
#    {gdecaso, atcach, vbraber, diegog, suchitel}@dc.uba.ar
#    LaFHIS - Departamento de Computación - Facultad de Ciencias Exactas y Naturales - Universidad de Buenos Aires
#    Pabellón I - Ciudad Universitaria - (C1428EGA) - Buenos Aires - Argentina
#    Tel: +54-11-4576-3390 eext. 704
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

from input.options.options import Options

from output.writer import Writer
class GraphvizWriter(Writer):

	def print_contents(self, fsm):
		
		self.print_header()
		print 
		
		print "digraph " + fsm.name + " {"
		print "\trankdir=LR;"
		
		# write initial state (if any)
		if fsm.initial_state != None:
			print "\tnode [shape = doublecircle", 
			if not Options.options.print_details:
				print ", label=\"\"",
			print "]; " + fsm.initial_state + ";"
			print "\tnode [shape = circle];"
		
			# calculate a mapping that stores every action that goes from s1 to s2
			# (so we can use a single arrow with a label that shows many actions)
			actions = {}
			for state in fsm.states:
				for transition in state.transitions:
					if state.name not in actions:
						state_actions = {}
						state_actions[transition.destination] = [transition]
						actions[state.name] = state_actions
					else:
						if transition.destination not in actions[state.name]:
							actions[state.name][transition.destination] = [transition]
						else:
							actions[state.name][transition.destination].append(transition)
	
			# see if there are any states of which their consistence is uncertain
			for state in fsm.states:
				print state.name + " [ label=\"" + self.state_label(state) + "\"];"

			# write each transition
			for state in actions:
				for destination in actions[state]:
					label = Options.options.gv_separator.join([self.transition_label(t) for t in actions[state][destination]])
					print "\t" + state + " -> " + destination + " [ label= \"" + label + "\" ];"
					
			if Options.options.print_information:
				print
				print "overlap=false"
				print "label=<<TABLE BORDER=\"0\"><TR><TD><BR/> <BR/> <BR/> <BR/> <BR/> <BR/></TD></TR><TR><TD>"
				print "<TABLE cellpadding=\"4\" cellspacing=\"0\">"
				from output.meta_information import MetaInformation
				for (attribute, value) in MetaInformation.info_attributes(fsm):
					print "\t<TR><TD BGCOLOR=\"#FFD39B\" ALIGN=\"RIGHT\"><FONT FACE=\"BOLD\">" + str(attribute) + "</FONT></TD><TD BGCOLOR=\"#FFECD3\" ALIGN=\"LEFT\">" + str(value) + "</TD></TR>"
				print "</TABLE></TD></TR></TABLE>>"
		print "}"
		
	def print_header(self):
		print "# autogenerated with contractor (http://lafhis.dc.uba.ar/contractor)"
		print "# transitions that may not exist are marked with '?'"
		print "# states that may not exist are marked with '?'"
		print "# transitions that may lead to a broken contract invariant are marked with '*'"

	def state_label(self, state):
		ret = ""
		if Options.options.print_details:
			ret += state.name
		if state.uncertain:
			ret += "?"
		return ret

	def transition_label(self, transition):
		ret = transition.label
		if transition.uncertain:
			ret += "?"
		if transition.violates_invariant:
			ret += "*"
		return ret
		
